@@ -41,6 +41,7 @@ def verilog(generator: Generator, optimize_if: bool = True,
             track_generated_definition: bool = False,
             contains_event: bool = False,
             lift_genvar_instances: bool = False,
+            fix_port_legality: bool = False,
             compile_to_verilog: bool = False):
     code_gen = _kratos.VerilogModule(generator.internal_generator)
     pass_manager = code_gen.pass_manager()
@@ -96,7 +97,6 @@ def verilog(generator: Generator, optimize_if: bool = True,
         pass_manager.add_pass("propagate_scope_variable")
         pass_manager.add_pass("inject_instance_ids")
         pass_manager.add_pass("inject_debug_break_points")
-        pass_manager.add_pass("inject_assert_fail_exception")
     if use_parallel:
         pass_manager.add_pass("hash_generators_parallel")
     else:
@@ -112,6 +112,9 @@ def verilog(generator: Generator, optimize_if: bool = True,
         pass_manager.add_pass("insert_pipeline_stages")
     if reorder_stmts:
         pass_manager.add_pass("sort_stmts")
+    # legality fix at the very end
+    if fix_port_legality:
+        pass_manager.add_pass("port_legality_fix")
 
     code_gen.run_passes()
 
