@@ -10,6 +10,7 @@ import _kratos
 import astor
 from _kratos import get_frame_local, StatementBlockType
 
+import kratos
 from .util import print_src
 
 
@@ -100,6 +101,12 @@ class StaticElaborationNodeForVisitor(ast.NodeTransformer):
                     return self.value
             return node
 
+        def visit_Assign(self, node):
+            if not isinstance(self.value, ast.Subscript):
+                # a little hacky to pass information around, but it works well
+                setattr(node, "value=" + self.target.id, self.value)
+            return self.generic_visit(node)
+
     class HasVar(ast.NodeVisitor):
         def __init__(self, target):
             self.has_target = False
@@ -293,39 +300,90 @@ class StaticElaborationNodeIfVisitor(ast.NodeTransformer):
         self.key_pair = []
 
     def __change_if_predicate(self, node):
-        if isinstance(node, ast.UnaryOp):
-            # notice that if the user uses `not var`, due to Python
-            # implementation, it will return True/False, we need to
-            # change that into r_not call
-            if isinstance(node.op, ast.Not):
-                target = node.operand
-                target_src = astor.to_source(target)
-                target_eval = eval(target_src, self.local)
-                if isinstance(target_eval, _kratos.Var):
-                    return ast.Call(func=ast.Attribute(value=target, attr="r_not", ctx=ast.Load()),
-                                    args=[], keywords=[], ctx=ast.Load())
-            else:
-                return node
-        elif not isinstance(node, ast.Compare):
-            return node
-        op = node.ops[0]
-        if not isinstance(op, ast.Eq):
-            return node
-        left = node.left
-        left_src = astor.to_source(left)
-        try:
-            left_val = eval(left_src, self.local)
-        except _kratos.exception.VarException:
-            left_val = None
+        class IfPredicateOpTransformer(ast.NodeTransformer):
+            def __init__(self, local, global_):
+                self.local = local
+                self.global_ = global_
 
-        if isinstance(left_val, _kratos.Var):
-            # change it into a function all
-            return ast.Call(func=ast.Attribute(value=left,
-                                               attr="eq",
-                                               ctx=ast.Load()),
-                            args=node.comparators,
-                            keywords=[],
-                            ctx=ast.Load)
+            def visit_UnaryOp(self, n: ast.UnaryOp):
+                # notice that if the user uses `not var`, due to Python
+                # implementation, it will return True/False, we need to
+                # change that into r_not call
+                if isinstance(n.op, ast.Not):
+                    target = n.operand
+                    target_src = astor.to_source(target)
+                    target_eval = eval(target_src, self.local, self.global_)
+                    if isinstance(target_eval, _kratos.Var):
+                        return ast.Call(func=ast.Attribute(value=target, attr="r_not", ctx=ast.Load()),
+                                        args=[], keywords=[], ctx=ast.Load())
+                else:
+                    return n
+
+            def visit_Compare(self, n: ast.Compare):
+                op = n.ops[0]
+
+                left = n.left
+                left_src = astor.to_source(left)
+                try:
+                    left_val = eval(left_src, self.local, self.global_)
+                except _kratos.exception.VarException:
+                    left_val = None
+
+                flip = not isinstance(left_val, _kratos.Var)
+
+                right = n.comparators
+                assert len(right) == 1, "Only size 1 comparators supported"
+                right_src = astor.to_source(right[0])
+                try:
+                    right_val = eval(right_src, self.local, self.global_)
+                except _kratos.exception.VarException:
+                    right_val = None
+
+                if not isinstance(right_val, _kratos.Var) and flip:
+                    # none of our business
+                    return n
+
+                if isinstance(op, ast.Eq):
+                    func_name = "eq"
+                elif isinstance(op, ast.NotEq):
+                    func_name = "__ne__"
+                elif isinstance(op, ast.Lt) and not flip:
+                    func_name = "__lt__"
+                elif isinstance(op, ast.Lt) and flip:
+                    func_name = "__gt__"
+                elif isinstance(op, ast.Gt) and not flip:
+                    func_name = "__gt__"
+                elif isinstance(op, ast.Gt) and flip:
+                    func_name = "__lt__"
+                elif isinstance(op, ast.LtE) and not flip:
+                    func_name = "__le__"
+                elif isinstance(op, ast.LtE) and flip:
+                    func_name = "__ge__"
+                elif isinstance(op, ast.GtE) and not flip:
+                    func_name = "__ge__"
+                elif isinstance(op, ast.GtE) and flip:
+                    func_name = "__le__"
+                else:
+                    return n
+
+                if flip:
+                    assert len(n.comparators) == 1
+                    value = n.comparators[0]
+                    args = [n.left]
+                else:
+                    value = n.left
+                    args = n.comparators
+                return ast.Call(func=ast.Attribute(value=value,
+                                                   attr=func_name,
+                                                   ctx=ast.Load()),
+                                args=args,
+                                keywords=[],
+                                ctx=ast.Load,
+                                lineno=n.lineno,
+                                col_offset=n.col_offset)
+
+        t = IfPredicateOpTransformer(self.local, self.global_)
+        node = t.visit(node)
         return node
 
     def visit_If(self, node: ast.If):
@@ -364,7 +422,7 @@ class StaticElaborationNodeIfVisitor(ast.NodeTransformer):
             # need to convert the logical operators to either reduced function calls, or
             # expression or
             if_test = LogicOperatorVisitor(self.local, self.global_, self.filename, self.scope_ln)
-            predicate = if_test.visit(node.test)
+            predicate = if_test.visit(predicate)
 
         expression = node.body
         else_expression = node.orelse
@@ -426,6 +484,15 @@ class AssignNodeVisitor(ast.NodeTransformer):
         args = node.targets[:] + [node.value]
         if self.debug:
             args.append(ast.Constant(value=node.lineno))
+        # loop through the attributes
+        variables = dir(node)
+        keywords = []
+        for var_name in variables:
+            if "value=" in var_name:
+                name = var_name.replace("value=", "")
+                value = getattr(node, var_name)
+                key = ast.keyword(arg=name, value=ast.Num(value))
+                keywords.append(key)
         new_node = ast.Expr(
             value=ast.Call(func=ast.Attribute(
                 value=ast.Name(id="scope",
@@ -433,7 +500,7 @@ class AssignNodeVisitor(ast.NodeTransformer):
                 attr="assign",
                 ctx=ast.Load()),
                 args=args,
-                keywords=[],
+                keywords=keywords,
                 lineno=node.lineno))
         self.target_node[node] = new_node
         return new_node
@@ -518,6 +585,12 @@ class GenVarLocalVisitor(ast.NodeTransformer):
             insert_keywords = False
             if attr.attr == "assign" and isinstance(attr.value, ast.Name) and attr.value.id == self.scope_name:
                 insert_keywords = True
+            # make sure we haven't inserted that keyword yet
+            if insert_keywords:
+                for entry in node.keywords:
+                    if entry.arg == self.key:
+                        insert_keywords = False
+                        break
             if insert_keywords:
                 keyword = ast.keyword(arg=self.key,
                                       value=ast.Str(s=self.value))
@@ -704,7 +777,9 @@ def transform_always_comb_ssa(ast_tree, gen, _locals):
 
     # need to add all the wires for each ref
     for v1, v2 in ssa.var_ref.items():
-        gen.wire(gen.vars[v1], gen.vars[v2], no_fn_ln=True)
+        # we ignore this final assignment since it's not considered in the source mapping
+        debug_attr = kratos.Attribute.create("debug-ignore")
+        gen.wire(gen.vars[v1], gen.vars[v2], no_fn_ln=True, attributes=debug_attr)
     return ast_tree
 
 
