@@ -9,18 +9,7 @@ namespace py = pybind11;
 
 void init_debug(py::module &m) {
     using namespace kratos;
-    m.def("inject_debug_break_points", &inject_debug_break_points)
-        .def("extract_debug_break_points", &extract_debug_break_points)
-        .def("inject_clock_break_points",
-             py::overload_cast<Generator *>(&inject_clock_break_points))
-        .def("inject_clock_break_points",
-             py::overload_cast<Generator *, const std::string &>(&inject_clock_break_points))
-        .def("inject_clock_break_points",
-             py::overload_cast<Generator *, const std::shared_ptr<Port> &>(
-                 &inject_clock_break_points))
-        .def("inject_assert_fail_exception", &inject_assert_fail_exception)
-        .def("inject_instance_ids", &inject_instance_ids)
-        .def("mock_hierarchy", &mock_hierarchy);
+    m.def("mock_hierarchy", &mock_hierarchy);
 
     py::class_<DebugDatabase>(m, "DebugDataBase")
         .def(py::init<>())
@@ -35,7 +24,6 @@ void init_debug(py::module &m) {
              py::overload_cast<const std::map<Generator *, std::map<std::string, std::string>> &>(
                  &DebugDatabase::set_variable_mapping),
              py::arg("mapping"))
-        .def("set_stmt_context", &DebugDatabase::set_stmt_context, py::arg("top"))
         // dump the database file
         .def("save_database",
              py::overload_cast<const std::string &, bool>(&DebugDatabase::save_database),
