@@ -456,7 +456,9 @@ void init_common_expr(py::class_<kratos::Var, ::shared_ptr<kratos::Var>> &class_
         .def("set_generator", &Var::set_generator)
         .def("set_size_param", &Var::set_size_param)
         .def_property("raw_type_param", &Var::get_raw_type_param, &Var::set_raw_type_param)
-        .def("__len__", [](const Var &var) { return var.size()[0]; });
+        .def("__len__", [](const Var &var) { return var.size()[0]; })
+        .def_property_readonly("sources", &Var::sources)
+        .def_property_readonly("sinks", &Var::sinks);
 
     def_attributes<py::class_<Var, ::shared_ptr<Var>>, Var>(class_);
 }
@@ -665,8 +667,15 @@ void init_expr(py::module &m) {
     // ternary op
     auto ternary_exp =
         py::class_<ConditionalExpr, ::shared_ptr<ConditionalExpr>, Expr>(m, "ConditionalExpr");
-    ternary_exp.def(py::init<const std::shared_ptr<Var> &, const std::shared_ptr<Var> &,
-                             const std::shared_ptr<Var> &>());
+    ternary_exp
+        .def(py::init<const std::shared_ptr<Var> &, const std::shared_ptr<Var> &,
+                      const std::shared_ptr<Var> &>())
+        .def(py::init(
+            py::overload_cast<const std::shared_ptr<Var> &, int64_t, const std::shared_ptr<Var> &>(
+                &ConditionalExpr::create)))
+        .def(py::init(
+            py::overload_cast<const std::shared_ptr<Var> &, const std::shared_ptr<Var> &, int64_t>(
+                &ConditionalExpr::create)));
     // function call expr
     auto call_Var =
         py::class_<FunctionCallVar, ::shared_ptr<FunctionCallVar>, Var>(m, "FunctionCallVar");
@@ -675,7 +684,9 @@ void init_expr(py::module &m) {
     m.def("constant", &constant, py::return_value_policy::reference);
     init_big_int(m);
 
-    m.def("mux", util::mux);
+    m.def("mux", py::overload_cast<Var &, Var &, Var &>(util::mux))
+        .def("mux", py::overload_cast<Var &, int64_t, Var &>(util::mux))
+        .def("mux", py::overload_cast<Var &, Var &, int64_t>(util::mux));
 
     py::class_<EnumVar, ::shared_ptr<EnumVar>, Var>(m, "EnumVar")
         .def("enum_type", &EnumVar::enum_type);
