@@ -67,7 +67,8 @@ void init_pass(py::module &m) {
         .def("auto_insert_clock_enable", &auto_insert_clock_enable)
         .def("auto_insert_sync_reset", &auto_insert_sync_reset)
         .def("change_property_into_stmt", &change_property_into_stmt)
-        .def("remove_event_stmts", &remove_event_stmts);
+        .def("remove_event_stmts", &remove_event_stmts)
+        .def("port_legality_fix", &port_legality_fix);
 
     auto manager = py::class_<PassManager>(pass_m, "PassManager", R"pbdoc(
 This class gives you the fined control over which pass to run and in which order.
@@ -115,6 +116,10 @@ by yourself to obtain the verilog code.)pbdoc");
         void visit(VarVarSlice *slice) override {
             PYBIND11_OVERLOAD(void, IRVisitor, visit, slice);
         }
+
+        void visit_external(Generator *generator) override {
+            PYBIND11_OVERLOAD(void, IRVisitor, visit, generator);
+        }
     };
 
     auto ast_visitor = py::class_<IRVisitor, PyIRVisitor>(pass_m, "IRVisitor");
