@@ -448,7 +448,8 @@ uint32_t num_size_decrease(Var *var) {
 }
 
 VarSlice::VarSlice(Var *parent, uint32_t high, uint32_t low)
-    : Var(parent->generator(), "", parent->var_width(), 1, parent->is_signed(), VarType::Slice),
+    // variable slice is always unsigned based on the spec
+    : Var(parent->generator(), "", parent->var_width(), 1, false, VarType::Slice),
       parent_var(parent),
       low(low),
       high(high),
@@ -675,6 +676,21 @@ std::string VarVarSlice::to_string() const {
     return ::format("{0}[{1}]", parent_var->to_string(), sliced_var_->to_string());
 }
 
+void resize_var(Expr *expr, uint32_t target_width, Var *var, bool left) {
+    if (var->type() == VarType::ConstValue) {
+        var->var_width() = target_width;
+    } else {
+        auto new_var = var->cast(VarCastType::Resize);
+        auto var_casted = new_var->as<VarCasted>();
+        var_casted->set_target_width(target_width);
+        if (left) {
+            expr->left = var_casted.get();
+        } else {
+            expr->right = var_casted.get();
+        }
+    }
+}
+
 Expr::Expr(ExprOp op, Var *left, Var *right)
     : Var(left->generator(), "", left->var_width(), left->size(), left->is_signed(),
           VarType::Expression),
@@ -682,30 +698,21 @@ Expr::Expr(ExprOp op, Var *left, Var *right)
       left(left),
       right(right) {
     assert(left->ir_node_kind() == IRNodeKind::VarKind);
-    if (right != nullptr && left->width() != right->width()) {
+    // notice that we allow shifting to have different size
+    if (right &&
+        (op == ExprOp::ShiftLeft || op == ExprOp::SignedShiftRight ||
+         op == ExprOp::LogicalShiftRight) &&
+        (left->width() > right->width())) {
+        // resize right
+        resize_var(this, left->width(), right, false);
+    } else if (right != nullptr && left->width() != right->width()) {
         // see if we can resize
         if (IterVar::safe_to_resize(left, right->width(), right->is_signed()) &&
             (right->type() != VarType::ConstValue && right->type() != VarType::Parameter &&
              right->type() != VarType::Iter)) {
-            // this is a hack
-            if (left->type() == VarType::ConstValue) {
-                left->var_width() = right->width();
-            } else {
-                // do a resize cast instead
-                auto new_left = left->cast(VarCastType::Resize);
-                auto left_casted = new_left->as<VarCasted>();
-                left_casted->set_target_width(right->width());
-                this->left = left_casted.get();
-            }
+            resize_var(this, right->width(), left, true);
         } else if (IterVar::safe_to_resize(right, left->width(), left->is_signed())) {
-            if (right->type() == VarType::ConstValue) {
-                right->var_width() = left->width();
-            } else {
-                auto new_right = right->cast(VarCastType::Resize);
-                auto right_casted = new_right->as<VarCasted>();
-                right_casted->set_target_width(left->width());
-                this->right = right_casted.get();
-            }
+            resize_var(this, left->width(), right, false);
         }
         if (this->left->width() != this->right->width())
             throw VarException(
@@ -1358,22 +1365,23 @@ void set_slice_var_parent(Var *&var, Var *target, Var *new_var, bool check_targe
     var = new_var_ptr.get();
 }
 
-void change_var_expr(const std::shared_ptr<Expr> &expr, Var *target, Var *new_var) {
+void change_var_expr(const std::shared_ptr<Expr> &expr, Var *target, Var *new_var,
+                     bool move_linked = true) {
     if (!new_var || !target) throw InternalException("Variable is NULL");
     if (expr->left->type() == VarType::Expression) {
-        change_var_expr(expr->left->as<Expr>(), target, new_var);
+        change_var_expr(expr->left->as<Expr>(), target, new_var, move_linked);
     }
     if (expr->right && expr->right->type() == VarType::Expression) {
-        change_var_expr(expr->right->as<Expr>(), target, new_var);
+        change_var_expr(expr->right->as<Expr>(), target, new_var, move_linked);
     }
 
     if (expr->left == target) {
         expr->left = new_var;
-        expr->left->move_linked_to(new_var);
+        if (move_linked) expr->left->move_linked_to(new_var);
     }
     if (expr->right && expr->right == target) {
         expr->right = new_var;
-        expr->right->move_linked_to(new_var);
+        if (move_linked) expr->right->move_linked_to(new_var);
     }
 
     // need to change the parent as well
@@ -1388,6 +1396,18 @@ void change_var_expr(const std::shared_ptr<Expr> &expr, Var *target, Var *new_va
         auto concat = expr->as<VarConcat>();
         concat->replace_var(target->shared_from_this(), new_var->shared_from_this());
     }
+    // need to deal with ternary op as well
+    if (expr->op == ExprOp::Conditional) {
+        auto ternary = expr->as<ConditionalExpr>();
+        auto *cond = ternary->condition;
+        if (cond->type() == VarType::Expression) {
+            change_var_expr(cond->as<Expr>(), target, new_var);
+        } else if (cond == target) {
+            ternary->condition = new_var;
+        } else if (cond->type() == VarType::Slice) {
+            set_slice_var_parent(cond, target, new_var, false);
+        }
+    }
 }
 
 void change_cast_parent(const std::shared_ptr<VarCasted> &var, Var *target, Var *new_var);
@@ -1639,6 +1659,20 @@ std::string ConditionalExpr::handle_name(kratos::Generator *scope) const {
                     right->handle_name(scope));
 }
 
+std::shared_ptr<ConditionalExpr> ConditionalExpr::create(const std::shared_ptr<Var> &condition,
+                                                         int64_t left,
+                                                         const std::shared_ptr<Var> &right) {
+    auto left_const = Const::constant(left, right->width(), right->is_signed()).shared_from_this();
+    return std::make_shared<ConditionalExpr>(condition, left_const, right);
+}
+
+std::shared_ptr<ConditionalExpr> ConditionalExpr::create(const std::shared_ptr<Var> &condition,
+                                                         const std::shared_ptr<Var> &left,
+                                                         int64_t right) {
+    auto right_const = Const::constant(right, left->width(), left->is_signed()).shared_from_this();
+    return std::make_shared<ConditionalExpr>(condition, left, right_const);
+}
+
 uint32_t PackedStructFieldDef::bitwidth() const {
     if (struct_) {
         return struct_->bitwidth();
@@ -2158,4 +2192,16 @@ std::shared_ptr<Expr> util::mux(Var &cond, Var &left, Var &right) {
     return expr;
 }
 
+std::shared_ptr<Expr> util::mux(Var &cond, int64_t left, Var &right) {
+    auto expr = ConditionalExpr::create(cond.shared_from_this(), left, right.shared_from_this());
+    cond.generator()->add_expr(expr);
+    return expr;
+}
+
+std::shared_ptr<Expr> util::mux(Var &cond, Var &left, int64_t right) {
+    auto expr = ConditionalExpr::create(cond.shared_from_this(), left.shared_from_this(), right);
+    cond.generator()->add_expr(expr);
+    return expr;
+}
+
 }  // namespace kratos
