@@ -96,7 +96,7 @@ void FSM::realize() {
     auto states = get_all_child_states(false);
     uint64_t num_states = states.size();
     if (!num_states) throw UserException(::format("FSM {0} is empty", fsm_name()));
-    uint32_t width = std::ceil(std::log2(num_states));
+    uint32_t width = std::max<uint32_t>(1u, std::ceil(std::log2(num_states)));
     // define a enum type
     std::map<std::string, uint64_t> raw_def;
     uint64_t count = 0;
@@ -221,7 +221,7 @@ std::shared_ptr<FunctionStmtBlock> FSM::get_func_def() {
     std::unordered_map<std::string, Var*> name_mapping;
     name_mapping.reserve(outputs_.size());
     // add outputs
-    for (auto const& var : outputs_) {
+    for (auto const& [var, _] : outputs_) {
         auto var_name = var->to_string() + "_value";
         func->input(var_name, var->width(), var->is_signed());
         name_mapping.emplace(var_name, var);
@@ -250,33 +250,41 @@ void FSM::generate_state_transition(
     if (!moore_) func_def = get_func_def();
     auto case_state_comb = std::make_shared<SwitchStmt>(current_state.shared_from_this());
     auto states = get_all_child_states(false);
+
+    auto add_debug_info_to_stmt = [this](std::shared_ptr<FunctionCallStmt>& func_stmt,
+                                         const FSMState* state) {
+        if (generator_->debug) {
+            if (func_stmt) {
+                add_debug_info(state, func_stmt);
+                func_stmt->fn_name_ln.emplace_back(std::make_pair(__FILE__, __LINE__));
+            }
+        }
+    };
     for (auto const& state : states) {
         auto const& state_name = state_name_mapping.at(state);
         // a list of if statements
         std::shared_ptr<IfStmt> if_ = nullptr;
         std::shared_ptr<IfStmt> top_if = nullptr;
-        std::vector<Var*> vars;
         auto transitions = state->transitions();
-        vars.reserve(transitions.size());
-        for (auto const& iter : transitions) vars.emplace_back(iter.first);
         // slide through condition
         bool has_slide_through = false;
-        if (vars.size() != 1 || vars[0] != nullptr) {
-            std::sort(vars.begin(), vars.end(), [=](const auto& lhs, const auto& rhs) {
-                return transitions.at(lhs)->name() < transitions.at(rhs)->name();
-            });
-        } else {
+        if (transitions.size() == 1 && transitions[0].first == nullptr) {
             has_slide_through = true;
         }
-        for (auto const& cond : vars) {
-            auto* next_fsm_state = transitions.at(cond);
-            if (!cond) {
+        std::optional<std::pair<Var *, FSMState *>> default_state;
+        for (auto const& [cond, next_fsm_state] : transitions) {
+            if (!cond && (transitions.size() == 1)) {
                 // direct transition
                 auto stmt = get_next_state_stmt(enum_def, next_state, state, next_fsm_state,
                                                 state_name_mapping);
                 case_state_comb->add_switch_case(enum_def.get_enum(state_name), stmt);
                 break;
+            } else if (!cond) {
+                // we have a default case here
+                default_state= std::make_pair(cond, next_fsm_state);
+                continue;
             }
+
             if (!if_) {
                 if_ = std::make_shared<IfStmt>(cond->shared_from_this());
                 auto stmt = get_next_state_stmt(enum_def, next_state, state, next_fsm_state,
@@ -289,12 +297,7 @@ void FSM::generate_state_transition(
                     if_->add_then_stmt(func_stmt);
                 }
                 top_if = if_;
-                if (generator_->debug) {
-                    if (func_stmt) {
-                        add_debug_info(state, func_stmt);
-                        func_stmt->fn_name_ln.emplace_back(std::make_pair(__FILE__, __LINE__));
-                    }
-                }
+                add_debug_info_to_stmt(func_stmt, state);
             } else {
                 auto new_if = std::make_shared<IfStmt>(cond->shared_from_this());
                 auto stmt = get_next_state_stmt(enum_def, next_state, state, next_fsm_state,
@@ -306,17 +309,26 @@ void FSM::generate_state_transition(
                     get_func_call_stmt(func_def, next_fsm_state, func_stmt);
                     new_if->add_then_stmt(func_stmt);
                 }
-                if (generator_->debug) {
-                    if (func_stmt) {
-                        add_debug_info(state, func_stmt);
-                        func_stmt->fn_name_ln.emplace_back(std::make_pair(__FILE__, __LINE__));
-                    }
-                }
+                add_debug_info_to_stmt(func_stmt, state);
                 if_->add_else_stmt(new_if);
                 if_ = new_if;
             }
         }
 
+        if (default_state) {
+            auto* next_fsm_state = default_state->second;
+            auto stmt = get_next_state_stmt(enum_def, next_state, state, next_fsm_state,
+                                            state_name_mapping);
+            if_->add_else_stmt(stmt);
+            // mealy machine need to add extra state transition outputs
+            std::shared_ptr<FunctionCallStmt> func_stmt = nullptr;
+            if (!moore_) {
+                get_func_call_stmt(func_def, next_fsm_state, func_stmt);
+                if_->add_then_stmt(func_stmt);
+            }
+            add_debug_info_to_stmt(func_stmt, state);
+        }
+
         if (!has_slide_through) {
             if (!top_if)
                 throw InternalException(
@@ -396,13 +408,12 @@ void FSM::generate_output(Enum& enum_def, EnumVar& current_state,
 
         std::vector<std::shared_ptr<Stmt>> stmts;
         auto const& output_values = state->output_values();
+        auto const& output_ordering = state->output_ordering();
         stmts.reserve(output_values.size());
         // sort the names
         std::vector<Var*> vars;
         vars.reserve(output_values.size());
-        for (auto const& iter : output_values) vars.emplace_back(iter.first);
-        std::sort(vars.begin(), vars.end(),
-                  [](auto& lhs, auto& rhs) { return lhs->name < rhs->name; });
+        for (auto* iter : output_ordering) vars.emplace_back(iter);
         for (auto const& output_var : vars) {
             auto* value = output_values.at(output_var);
             if (value && value != output_var) {
@@ -429,12 +440,16 @@ void FSM::generate_output(Enum& enum_def, EnumVar& current_state,
     output_comb->add_stmt(output_case_comb);
 }
 
-void FSM::output(const std::string& var_name) {
+void FSM::output(const std::string& var_name) { output(var_name, nullptr); }
+
+void FSM::output(const std::string& var_name, const std::shared_ptr<Var>& default_) {
     auto var = generator_->get_var(var_name);
-    output(var);
+    output(var, default_);
 }
 
-void FSM::output(const std::shared_ptr<Var>& var) {
+void FSM::output(const std::shared_ptr<Var>& var) { output(var, nullptr); }
+
+void FSM::output(const std::shared_ptr<Var>& var, const std::shared_ptr<Var>& default_) {
     if (!var) throw UserException(::format("var not found in {0}", generator_->instance_name));
     // very strict checking of ownership
     if (var->parent() != generator_) {
@@ -442,7 +457,7 @@ void FSM::output(const std::shared_ptr<Var>& var) {
             throw VarException("FSM output has to be scoped inside the top-level of generator",
                                {var.get()});
     }
-    outputs_.emplace(var.get());
+    outputs_.emplace(var.get(), default_.get());
 }
 
 std::shared_ptr<FSMState> FSM::add_state(const std::string& name) {
@@ -522,17 +537,10 @@ std::string FSM::dot_graph() {
     stream << ::endl;
     // state transition
     for (auto const& state : states) {
-        auto transitions = state->transitions();
+        auto const &transitions = state->transitions();
         auto state_name = state->handle_name();
         // deterministic sorting
-        std::vector<Var*> conds;
-        conds.reserve(transitions.size());
-        for (auto const& iter : transitions) conds.emplace_back(iter.first);
-        std::sort(conds.begin(), conds.end(), [](auto const& lhs, auto const& rhs) {
-            return lhs->to_string() < rhs->to_string();
-        });
-        for (auto const& cond : conds) {
-            auto* next_state = transitions.at(cond);
+        for (auto [cond, next_state] : transitions) {
             if (cond) {
                 stream << indent
                        << ::format("{0}    ->  {1} [ label = \"{2}\" ];", state_name,
@@ -561,7 +569,7 @@ std::string FSM::output_table() {
     // sort the outputs
     std::vector<Var*> outputs;
     outputs.reserve(outputs_.size());
-    for (auto const& var : outputs_) outputs.emplace_back(var);
+    for (auto const& iter : outputs_) outputs.emplace_back(iter.first);
     std::sort(outputs.begin(), outputs.end(),
               [](const auto& lhs, const auto& rhs) { return lhs->to_string() < rhs->to_string(); });
     // write the header
@@ -657,21 +665,28 @@ void FSMState::next(const std::shared_ptr<FSMState>& next_state, const std::shar
                                      parent->fsm_name()));
     }
 
+    auto find_match = [this](Var *ptr) -> bool {
+        auto it = std::find_if(transitions_.begin(), transitions_.end(), [ptr](auto const &pair) {
+            return pair.first == ptr;
+        });
+        return it != transitions_.end();
+    };
+
     if (!cond) {
-        transitions_.emplace(nullptr, state_ptr);
+        transitions_.emplace_back(std::make_pair(nullptr, state_ptr));
     } else {
-        if (transitions_.find(nullptr) != transitions_.end()) {
+        if (find_match(nullptr)) {
             // we have a slide through
             throw UserException("Unconditional transition has been assign to " + name_);
         }
         auto* ptr = cond.get();
         if (cond->width() != 1) throw VarException("Condition has to be a boolean value", {ptr});
 
-        if (transitions_.find(ptr) != transitions_.end()) {
+        if (find_match(ptr)) {
             throw ::runtime_error(::format("{0} has been added to FSM {1}-{2} already",
                                            ptr->to_string(), parent_->fsm_name(), name_));
         }
-        transitions_.emplace(ptr, state_ptr);
+        transitions_.emplace_back(std::make_pair(ptr, state_ptr));
     }
 }
 
@@ -689,6 +704,7 @@ void FSMState::output(const std::shared_ptr<Var>& output_var,
         throw VarException(::format("{0} already has specified output"),
                            {output_values_.at(output)});
     }
+    output_ordering_.emplace_back(output);
     output_values_.emplace(output, value);
 }
 
@@ -711,9 +727,14 @@ void FSMState::output(const std::shared_ptr<Var>& output_var, const std::shared_
 
 void FSMState::check_outputs() {
     auto outputs = parent_->outputs();
-    for (auto const& output : outputs) {
+    for (auto const& [output, default_] : outputs) {
         if (output_values_.find(output) == output_values_.end()) {
-            throw VarException(::format("{0} not specified", output->to_string()), {output});
+            if (!default_) {
+                throw VarException(::format("{0} not specified", output->to_string()), {output});
+            } else {
+                // insert it to the output values
+                this->output(output->shared_from_this(), default_->shared_from_this());
+            }
         }
     }
     // the other way, this is to ensure a bijection
