@@ -42,6 +42,18 @@ Generator *Stmt::generator_parent() const {
     return dynamic_cast<Generator *>(p);
 }
 
+Stmt *Stmt::pre_stmt() const {
+    if (!parent_ || parent_->ir_node_kind() != IRNodeKind::StmtKind) return nullptr;
+    auto *stmt_parent = reinterpret_cast<Stmt*>(parent_);
+    if (stmt_parent->type_ != StatementType::Block) return nullptr;
+    auto const &block = stmt_parent->as<StmtBlock>();
+    auto index = block->index_of(this);
+    if (index != 0 && index < block->size()) {
+        return (*block)[index - 1].get();
+    }
+    return nullptr;
+}
+
 void Stmt::set_scope_context(const std::map<std::string, std::pair<bool, std::string>> &context) {
     scope_context_ = context;
 }
@@ -187,7 +199,7 @@ IfStmt::IfStmt(std::shared_ptr<Var> predicate)
     // just to add the sinks
     predicate_stmt_ =
         predicate_->generator()->get_auxiliary_var(predicate_->width())->assign(predicate_);
-    predicate_stmt_->set_parent(nullptr);
+    predicate_stmt_->set_parent(this);
 }
 
 void IfStmt::set_predicate(const std::shared_ptr<Var> &var) {
@@ -196,7 +208,7 @@ void IfStmt::set_predicate(const std::shared_ptr<Var> &var) {
     predicate_ = var;
     predicate_stmt_ =
         predicate_->generator()->get_auxiliary_var(predicate_->width())->assign(predicate_);
-    predicate_stmt_->set_parent(nullptr);
+    predicate_stmt_->set_parent(this);
 }
 
 void IfStmt::add_then_stmt(const std::shared_ptr<Stmt> &stmt) {
@@ -256,7 +268,7 @@ void IfStmt::set_parent(IRNode *node) {
     Stmt::set_parent(node);
     then_body_->set_parent(this);
     else_body_->set_parent(this);
-    predicate_stmt_->set_parent(nullptr);
+    predicate_stmt_->set_parent(this);
 }
 
 IRNode *IfStmt::get_child(uint64_t index) {
@@ -473,7 +485,14 @@ SwitchStmt::SwitchStmt(Var &target)
 
     // just to add the sinks
     target_stmt_ = target.generator()->get_auxiliary_var(target.width())->assign(target);
-    target_stmt_->set_parent(nullptr);
+    target_stmt_->set_parent(this);
+}
+
+void SwitchStmt::set_target(const std::shared_ptr<Var> &target) {
+    target_stmt_->clear();
+    target_ = target;
+    target_stmt_ = target->generator()->get_auxiliary_var(target->width())->assign(target);
+    target_stmt_->set_parent(this);
 }
 
 void SwitchStmt::set_parent(IRNode *parent) {
@@ -728,7 +747,8 @@ BuiltInFunctionStmtBlock::BuiltInFunctionStmtBlock(Generator *parent,
     if (known_functions_.empty()) {
         // initialize
         // all the known functions should be synthesizable
-        known_functions_ = {{"clog2", 32}};
+        known_functions_ = {
+            {"clog2", 32}, {"countones", 1}, {"onehot", 1}, {"onehot0", 1}, {"isunknown", 1}};
     }
     if (known_functions_.find(function_name) == known_functions_.end()) {
         std::string known_functions;
