@@ -4,6 +4,17 @@ import _kratos
 import sqlite3
 import tempfile
 import os
+import pytest
+import json
+
+
+def get_line_num(txt):
+    with open(__file__) as f:
+        lines = f.readlines()
+    for i, line in enumerate(lines):
+        if line.strip() == txt:
+            return i + 1
+    return 0
 
 
 def test_db_dump():
@@ -15,12 +26,21 @@ def test_db_dump():
         debug_db = os.path.join(temp, "debug.db")
         # hashing and generate verilog
         verilog(mod, insert_debug_info=True, debug_db_filename=debug_db)
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        c.execute("SELECT * from breakpoint")
-        result = c.fetchall()
-        assert len(result) == 1
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+    assert db["generator"] == "kratos"
+    assert db["top"] == mod.name
+    mod_def = db["table"][0]
+    assert mod_def["name"] == mod.name
+    assert len(mod_def["scope"]) == 1
+    comb = mod_def["scope"][0]
+    assert comb["filename"] == __file__
+    assert len(comb["scope"]) == 1
+    assign = comb["scope"][0]
+    ref_line = get_line_num('comb.add_stmt(mod.var("a", 1).assign(mod.var("b", 1)))')
+    assert assign["line"] == ref_line
+    var = assign["variable"]
+    assert var["name"] == "a"
 
 
 def test_debug_mock():
@@ -96,18 +116,43 @@ def test_seq_debug():
         filename = os.path.join(temp, "test.sv")
         verilog(mod, filename=filename, insert_debug_info=True,
                 debug_db_filename=debug_db)
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        c.execute("SELECT * FROM breakpoint WHERE id=7")
-        result = c.fetchall()
-        assert len(result) == 1
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+    # it should have two scopes
+    mod_db = db["table"][0]
+    assert mod_db["name"] == mod.name
+    scopes = mod_db["scope"]
+    assert len(scopes) == 2
+    variables = {}
+    for v in db["variables"]:
+        variables[v["id"]] = v
+    for i in range(2):
+        code = scopes[i]
+        decl = code["scope"][0]
+        assert decl["type"] == "decl"
+        var = db["variables"][-1]
+        # this one is i, which should be compressed
+        v_id = decl["variable"]
+        assert v_id in variables
+        var = variables[v_id]
+        assert var["name"] == "i"
+        assert var["value"] == "3"
+        if_stmt = code["scope"][1]
+        assert len(if_stmt["scope"]) == 2
+        for stmt in if_stmt["scope"]:
+            assert len(stmt["scope"]) == 2
+            # two assignments
+            for a in stmt["scope"]:
+                assert a["type"] == "assign"
+    # test out gen var
+    assert len(mod_db["variables"]) == 1
+    assert mod_db["variables"][0]["name"] == "self.in_"
 
 
 def test_context():
     class Mod(Generator):
         def __init__(self, width):
-            super().__init__("mod", True)
+            super().__init__("mod", debug=True)
             in_ = self.input("in", width)
             out = self.output("out", width)
             sel = self.input("sel", width)
@@ -124,18 +169,25 @@ def test_context():
                         out[i] = 1
             self.add_always(code)
 
-    mod = Mod(4)
+    w = 4
+    mod = Mod(w)
     with tempfile.TemporaryDirectory() as temp:
         debug_db = os.path.join(temp, "debug.db")
         filename = os.path.join(temp, "test.sv")
         verilog(mod, filename=filename, insert_debug_info=True,
                 debug_db_filename=debug_db)
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        c.execute("SELECT * FROM context_variable")
-        variables = c.fetchall()
-        assert len(variables) > 20
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+        mod_db = db["table"][0]
+        code = mod_db["scope"][-1]
+        else_blk = code["scope"][-1]["scope"][-1]["scope"]
+        assert len(else_blk) == (w * 2)
+        values = set()
+        for entry in else_blk:
+            if entry["type"] == "decl" and entry["variable"]["name"] == "i":
+                values.add(int(entry["variable"]["value"]))
+        for i in range(w):
+            assert i in values
 
 
 def test_clock_interaction():
@@ -168,6 +220,7 @@ def test_clock_interaction():
                 debug_db_filename=debug_db)
 
 
+@pytest.mark.skip(reason="DPI-based assertion no longer supported")
 def test_assert():
     from kratos import assert_
     mod = Generator("mod", True)
@@ -215,20 +268,18 @@ def test_wire():
         filename = os.path.join(temp, "test.sv")
         verilog(mod, filename=filename, insert_debug_info=True,
                 debug_db_filename=debug_db)
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        c.execute("SELECT * FROM breakpoint")
-        assert len(c.fetchall()) == 1
-        c.execute("""SELECT variable.value FROM variable, context_variable
-                     WHERE context_variable.name = ? AND context_variable.variable_id = variable.id""", "a")
-        v = int(c.fetchone()[0])
-        assert v == a
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+        assign = db["table"][0]["scope"][0]["scope"][-1]
+        assert assign["type"] == "assign"
+        assert assign["variable"]["name"] == "out"
+        line = get_line_num("mod.wire(out_, in_)")
+        assert assign["line"] == line
 
 
-def test_inst_id():
-    def create_mod():
-        m = Generator("mod", True)
+def test_multiple_definitions():
+    def create_mod(idx):
+        m = Generator("mod" + str(idx), True)
         in_ = m.input("in", 1)
         out = m.output("out", 1)
         comb = m.combinational()
@@ -237,7 +288,7 @@ def test_inst_id():
     mod = Generator("parent", True)
     input_ = mod.input("in", 1)
     output = mod.output("out", 1)
-    mods = [create_mod() for _ in range(2)]
+    mods = [create_mod(i) for i in range(2)]
     expr = None
     for i, m_ in enumerate(mods):
         mod.add_child("mod{0}".format(i), m_)
@@ -252,12 +303,9 @@ def test_inst_id():
         debug_db = os.path.join(temp, "debug.db")
         verilog(mod, insert_debug_info=True, debug_db_filename=debug_db,
                 optimize_passthrough=False)
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        c.execute("SELECT * FROM instance")
-        res = c.fetchall()
-        assert len(res) == 3
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+        assert len(db["table"]) == 3
 
 
 def test_empty():
@@ -314,22 +362,31 @@ def test_array_packed():
     with tempfile.TemporaryDirectory() as temp:
         debug_db = os.path.join(temp, "debug.db")
         verilog(mod, debug_db_filename=debug_db, insert_debug_info=True)
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        c.execute("SELECT variable.value, generator_variable.name FROM variable, generator_variable WHERE variable.id = generator_variable.variable_id")
-        vars_ = c.fetchall()
-        c.execute("SELECT variable.value, context_variable.name FROM variable, context_variable WHERE variable.id = context_variable.variable_id")
-        vars_ += c.fetchall()
-        correct_struct, correct_array, correct_self = False, False, False
-        for value, name in vars_:
-            if value == "a[1][3]" and name == "aa.1.3":
-                correct_array = True
-            if value == "s.read" and name == "ss.read":
-                correct_struct = True
-            if "self.sss" in name:
-                correct_self = True
-        assert correct_array and correct_struct, correct_self
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+    mod_db = db["table"][0]
+    mod_variables = mod_db["variables"]
+    assert len(mod_variables) == 2
+    assert mod_variables[0]["name"] == "self.sss.read"
+    assert mod_variables[1]["name"] == "self.sss.data"
+    assert mod_variables[0]["value"] == "s.read"
+    assert mod_variables[1]["value"] == "s.data"
+
+    first_assign = mod_db["scope"][0]
+    names = set()
+    values = set()
+    for entry in first_assign["scope"]:
+        if entry["type"] == "assign":
+            var = entry["variable"]
+            names.add(var["name"])
+            values.add(var["value"])
+
+    for i in range(4):
+        for j in range(2):
+            name = "a.{0}.{1}".format(j, i)
+            value = "a[{0}][{1}]".format(j, i)
+            assert name in names
+            assert value in values
 
 
 def test_multiple_instance():
@@ -368,10 +425,6 @@ def test_multiple_instance():
     with tempfile.TemporaryDirectory() as temp:
         filename = os.path.join(temp, "test.sv")
         verilog(top, filename=filename, insert_debug_info=True, debug_fn_ln=True)
-        # check the id and line info
-        with open(filename) as f:
-            content = f.read()
-            assert ".KRATOS_INSTANCE_ID(32'h2))" in content
         # make sure that the line is mapped correctly
         with open(__file__) as f:
             lines = f.readlines()
@@ -393,13 +446,6 @@ def test_multiple_instance():
         assert count == num_instance * (num_instance + 3) / 2
 
 
-def get_line(line):
-    with open(__file__) as f:
-        lines = f.readlines()
-        lines = [l.rstrip() for l in lines]
-    return lines.index(line) + 1
-
-
 def test_ssa_debug():
     mod = Generator("mod", debug=True)
     a = mod.var("a", 4)
@@ -419,7 +465,7 @@ def test_ssa_debug():
         if a == 1:
             b = 1
         else:
-            b = 0
+            b = 0   # test_ssa_debug
 
     mod.add_always(logic1, ssa_transform=True)
     mod.add_always(logic2)
@@ -427,28 +473,53 @@ def test_ssa_debug():
     with tempfile.TemporaryDirectory() as temp:
         debug_db = os.path.join(temp, "debug.db")
         verilog(mod, insert_debug_info=True, debug_db_filename=debug_db, ssa_transform=True)
-        # assert the line number tracking
-        conn = sqlite3.connect(debug_db)
-        c = conn.cursor()
-        idx = get_line("                a = a + i")
-        c.execute("SELECT * FROM breakpoint WHERE line_num=?", (idx,))
-        result = c.fetchall()
-        assert len(result) == loop_size
-        assert "a_4" in result[0][-1]
-        # check the context variable
-        c.execute("SELECT * FROM context_variable WHERE context_variable.name = 'i'")
-        result = c.fetchall()
-        assert len(result) == loop_size
-        # check the assignment
-        c.execute("SELECT * FROM assignment WHERE assignment.name = 'a'")
-        result = c.fetchall()
-        assert result[0][1] == "a_0"
-        assert len(result) == (loop_size + 2)
-        c.execute("SELECT * FROM assignment WHERE assignment.name = 'b'")
-        result = c.fetchall()
-        assert len(result) == 2
-        conn.close()
+        with open(debug_db) as f:
+            db = json.load(f)
+    mod_db = db["table"][0]
+    logic1_db = mod_db["scope"][0]["scope"]
+    assert len(logic1_db) == (4 + 2 * (loop_size + 1))
+    # a_0 = 0
+    # 4x (a_i + i)
+    idx = 0
+    for entry in logic1_db[4:13]:
+        var = entry["variable"]
+        name = var["name"]
+        value = var["value"]
+        if name == "i":
+            assert value == str(idx)
+            idx += 1
+        else:
+            assert name == "a"
+            assert value == "a_{0}".format(idx)
+    # enable conditions
+    line = get_line_num("b = 0   # test_ssa_debug")
+    logic2_db = mod_db["scope"][1]["scope"]
+    last_assign = logic2_db[-1]["scope"][-1]["scope"][0]
+    assert last_assign["type"] == "assign"
+    assert last_assign["condition"] == "!(a == 4'h1)"
+    assert last_assign["line"] == line
+
+
+def test_assign_slice():
+    mod = Generator("mod", debug=True)
+    a = mod.var("a", 16, size=[2])
+    b = mod.input("b", 1)
+    c = mod.input("c", 16)
+    mod.wire(c, a[b])
+
+    with tempfile.TemporaryDirectory() as temp:
+        debug_db = os.path.join(temp, "debug.db")
+        verilog(mod, insert_debug_info=True, debug_db_filename=debug_db)
+        with open(debug_db) as f:
+            db = json.load(f)
+    mod_db = db["table"][0]
+    assert mod_db["name"] == "mod"
+    assigns = mod_db["scope"][0]["scope"][3:]
+    assert len(assigns) == 2
+    for i in range(2):
+        assert assigns[i]["variable"]["name"] == "a.{0}".format(i)
+        assert assigns[i]["variable"]["value"] == "a[{0}]".format(i)
 
 
 if __name__ == "__main__":
-    test_ssa_debug()
+    test_assign_slice()
