@@ -1,5 +1,6 @@
+import kratos
 from kratos import Generator, signed, unsigned, verilog
-from kratos.util import reduce_or, Const, const
+from kratos.util import reduce_or, Const, const, countones
 import _kratos.exception
 
 
@@ -246,5 +247,32 @@ def test_resize_const():
     assert str(c) == "-2'h1"
 
 
+def test_bit_task():
+    mod = Generator("mod")
+    a = mod.var("a", 10)
+    b = countones(a)
+    assert str(b) == "$countones (a)"
+
+
+def test_slice_signed():
+    from kratos import signed
+    mod = Generator("mod")
+    a = mod.var("a", width=15, size=5, explicit_array=True, is_signed=True)
+    assert a.signed
+    assert not a[0].signed
+    s = signed(a[1])
+    assert str(s) == "signed'(a[1])"
+
+
+def test_infer_const():
+    mod = Generator("mod")
+    a = mod.var("a", 4)
+    b = mod.var("b", 1)
+    expr = a == 1
+    assert str(expr) == "a == 4'h1"
+    expr = kratos.mux(b, 0, a)
+    assert str(expr) == "b ? 4'h0: a"
+
+
 if __name__ == "__main__":
-    test_resize_const()
+    test_infer_const()
