@@ -940,6 +940,26 @@ def test_fsm_mealy(check_gold):
     check_gold(mod, "test_fsm_mealy", optimize_if=False)
 
 
+def test_fsm_default_output():
+    mod = Generator("mod", debug=True)
+    out_ = mod.output("out", 2)
+    in_ = mod.input("in", 2)
+    # fsm requires a clk and async rst
+    mod.clock("clk")
+    mod.reset("rst")
+    fsm = mod.add_fsm("Color")
+    fsm.output(out_, const(2, 2))
+    # add states
+    red = fsm.add_state("Red")
+    blue = fsm.add_state("Blue")
+    red.next(blue, in_ == 0)
+    blue.next(red, in_ == 1)
+    blue.output(out_, 1)
+    fsm.set_start_state("Red")
+    src = verilog(mod)["mod"]
+    assert "out = 2'h2;" in src
+
+
 def test_function(check_gold):
     from kratos.func import function
 
@@ -2135,12 +2155,12 @@ def test_ssa_transform(check_gold):
     # which corresponds to a = 5
     # notice that a should be pointing to a = b + a, since it's the last
     # time a gets assigned
-    stmt = mod.get_stmt_by_index(6)
+    stmt = mod.get_stmt_by_index(3)[3]
     scope = stmt.scope_context
     is_var, a_mapping = scope["a"]
     assert is_var
     # this is assign a_2 = b + a_1;
-    stmt = mod.get_stmt_by_index(5)
+    stmt = mod.get_stmt_by_index(3)[2]
     assert str(a_mapping) == str(stmt.left)
 
     # test enable table extraction
@@ -2328,6 +2348,27 @@ def test_struct_of_struct(check_gold):
     check_gold(mod, "test_struct_of_struct")
 
 
+def test_external_visit():
+    from kratos import IRVisitor
+    p = Generator("parent")
+    c = Generator("Child")
+    c.external = True
+    p.add_child("inst", c)
+
+    class ExternalVisitor(IRVisitor):
+        def __init__(self):
+            super(ExternalVisitor, self).__init__()
+            self.value = False
+
+        def visit(self, node):
+            if node.name == "Child":
+                self.value = True
+
+    v = ExternalVisitor()
+    v.visit_root(p.internal_generator)
+    assert v.value
+
+
 if __name__ == "__main__":
-    from conftest import check_gold_fn
-    test_struct_of_struct(check_gold_fn)
+    from conftest import check_gold_fn, check_file_fn
+    test_ssa_transform(check_gold_fn)
