@@ -1,3 +1,4 @@
+import kratos
 from kratos import Generator, verilog, const, always_comb, Interface
 import platform
 import pytest
@@ -228,5 +229,115 @@ def test_regression_struct_struct_access():
     verilog(mod)
 
 
+def test_regression_decouple_wire():
+    # test reported from Jake Ke
+    from kratos import ternary
+
+    class Child(Generator):
+        def __init__(self):
+            super().__init__("child")
+            self._a = self.input("a", 1)
+            self._b = self.output("b", 1)
+            self.wire(self._b, 1)
+
+    class Parent(Generator):
+        def __init__(self):
+            super().__init__("parent")
+            self._a_in = self.input("a_in", 1)
+
+            child_inst = Child()
+            self.add_child("child_inst", child_inst, a=self._a_in)
+
+            self._c = self.var("c", 1)
+            self.add_code(self.some_comb)
+
+            self._d = self.var("d", 1)
+            # self.wire(self._d, self._c + self["child_inst"].ports.b) # this is OK for some reason
+            # I would like to get the generated wire, child_inst_b
+            self.wire(self._d, ternary(self["child_inst"].ports.b, self._c, self._c))
+            self._e = self.var("e", 1)
+            self.add_always(self.if_comb)
+            self._f = self.var("f", 1)
+            self.wire(self._f, self["child_inst"].ports.a == 0)
+
+        @always_comb
+        def some_comb(self):
+            self._c = self["child_inst"].ports.a # I would like to get a_in
+
+        @always_comb
+        def if_comb(self):
+            if self["child_inst"].ports.a:
+                self._e = 1
+            else:
+                self._e = 0
+
+    mod = Parent()
+    src = verilog(mod, fix_port_legality=True)["parent"]
+    assert "assign d = child_inst_b ? c: c;" in src
+    assert "c = a_in;" in src
+    assert "if (a_in) begin" in src
+    assert "assign f = a_in == 1'h0;" in src
+
+
+def test_one_state_fsm():
+    # example provided by Max
+    class OneStateFSM(Generator):
+        def __init__(self, debug: bool = False, is_clone: bool = False, internal_generator=None):
+            super().__init__("onestate", debug, is_clone, internal_generator)
+
+            self._clk = self.clock("clk")
+            self._rst_n = self.reset("rst_n")
+
+            self.boring_fsm = self.add_fsm("boring_fsm", reset_high=False)
+            START = self.boring_fsm.add_state("START")
+
+            self._boring_var = self.var("boring_var", 1)
+            START.next(START, None)
+            self.boring_fsm.output(self._boring_var)
+            START.output(self._boring_var, 1)
+
+            self.boring_fsm.set_start_state(START)
+
+    mod = OneStateFSM()
+    verilog(mod)
+
+
+def test_and():
+    mod = Generator("gen", debug=True)
+    a = mod.var("a", 4)
+    b = mod.var("b", 10)
+    c = mod.var("c", 1)
+
+    @always_comb
+    def code():
+        c = 1
+        for i in range(4):
+            if (i < a) and (b == const(1, 10)):
+                c = 0
+
+    mod.add_code(code)
+
+    src = verilog(mod)["gen"]
+    # notice the flipping
+    assert "(a > 4'h1) && (b == 10'h1)" in src
+
+
+def test_static_if():
+    mod = Generator("gen")
+    a = mod.output("a", 1)
+    b = 1
+
+    @always_comb
+    def code():
+        if b == 1:
+            a = 1
+        else:
+            a = 0
+
+    mod.add_always(code)
+    src = verilog(mod)["gen"]
+    assert "a = 1'h1" in src
+
+
 if __name__ == "__main__":
-    test_regression_packed_struct_array()
+    test_static_if()
